#
# Author: Kapil Thangavelu <kvthan@wm.edu>
# CVS: $Id: SQLEngine.py,v 1.5 2001/09/28 00:42:03 ender Exp $
# License: GPL


from OFS.Folder import Folder
from OFS.ObjectManager import ObjectManager
from AccessControl import ClassSecurityInfo
from Globals import DTMLFile
import Globals

from copy import copy

#from os.path import getmtime

from EngineConfig import config

def manage_addSQLEngine(self,
                        id,
                        title,
                        conn_id,
                        REQUEST=None):
    """ """
    ob = SQLEngine(id, title)
    self._setObject(id, ob)

    ob= ob.__of__(self)
    ob.manage_changeProperties(REQUEST)
    
    if REQUEST is not None:
        self.manage_main(self, REQUEST)
    
manage_addSQLEngineForm = DTMLFile('dtml/manage_addSQLEngineForm',globals())

class SQLEngine(Folder):
    """
    Makes SQL Statements loaded from XML
    available to Zope w/ abstracted interface. """

    meta_type = 'SQL Engine'

    security = ClassSecurityInfo()

    engine_manage_options = (
                             
                             {'label':'Sources',
                              'action':'manage_editSources',
                              'help':('SQLEngine', 'sources.stx')},
                             {'label':'Resources',
                              'action':'manage_editResources',
                              'help':('SQLEngine', 'resources.stx')}
                             )
    
    manage_options=engine_manage_options+Folder.manage_options


    _properties = (
                    {'id':'title', 'type':'string', 'mode':'w',
                     'default':''},
                    {'id':'conn_id', 'type':'string', 'mode':'w',
                     'default':''},
                   )


    def __init__(self, id, title):
        self.id = id
        self.title = title
        self._sources_map_resources = {}
        self._query_names = {}
        self._query_resource_modtimes = {}
        self._resource_map_queries = {}


    index_html = None
    __call__ = None

    # management interfaces
    security.declareProtected('Manage SQLEngine', 'manage_editSources')
    manage_editSources=DTMLFile('dtml/manage_editSources', globals())

    security.declareProtected('Manage SQLEngine', 'manage_editResources')
    manage_editResources=DTMLFile('dtml/manage_editResources', globals())
    
    security.declareProtected('Manage SQLEngine', 'manage_editQueries') 
    manage_editQueries=DTMLFile('dtml/manage_editQueries', globals())
                                  
    ## management support functions
    
    # source manage support
    security.declareProtected('Manage SQLEngine', 'manage_getSources')
    def manage_getSources(self):
        """ publisher string"""       
        return self._sources_map_resources.keys()

    security.declareProtected('Manage SQLEngine', 'manage_addSource')
    def manage_addSource(self, source):
        """ publisher string"""
        if type(source) != type ('s'):
            raise SyntaxError ("Source should be a string")
        self._sources_map_resources={}
        self._sources_map_resources[source]=[]
    
    security.declareProtected('Manage SQLEngine', 'manage_removeSource')
    def manage_removeSource(self, source): 
        """ publisher string"""        
        if self._sources_map_resources.has_key(source) == 1:
            # remove queries and resources
            resources = self._sources_map_resources[source]
            
            for r in resources:
                queries = self._resource_map_queries[r]
                qobjs = []
                for q in queries:
                    qobjs.append(self._query_names[q])
                    del self._query_names[q]
                    
                self.manage_delObjects(qobjs)
                del self._resource_map_queries[r]

            del self._sources_map_resources[source]                            
                
            self._p_changed=1

    # resource manage support
    security.declareProtected('Manage SQLEngine', 'manage_getResources')
    def manage_getResources(self):
        """ publisher string"""        
        return self._resource_map_queries.keys()

    security.declareProtected('Manage SQLEngine', 'manage_getResourcesForSource')    
    def manage_getResourcesForSource(self, source):
        """ publisher string"""                

        return copy(self._sources_map_resources[source])

    security.declareProtected('Manage SQLEngine', 'manage_getResourceModTime')    
    def manage_getResourceModTime(self, resource):
        """ publisher string"""                
        return self._query_resource_modtimes[resource]
    
    # add/remove resources by adding sources
    
    # query manage support
    security.declareProtected('Manage SQLEngine', 'manage_getQueries')    
    def manage_getQueries(self):
        """ publisher string"""                
        return self._query_names.keys()
    
    security.declareProtected('Manage SQLEngine',
                              'manage_getQueriesForResource')            
    def manage_getQueriesForResource(self, resource):
        """ publisher string"""                
        return copy(self._resource_map_queries[resource])


    # functional interface
    security.declareProtected('Use Database Methods','execute')
    def execute(self, query_name, args={}):
        """ maps a query name onto a sql method and executes it """

        qid = self._query_names.get(query_name,None)
        if not qid: raise "Query Name %s Not Found"%query_name

        qobj = getattr(self,qid,None)
        if not qobj: raise "No QueryObject Found %s %s"%(str(qid),query_name)

        return qobj(args)

    # guts
    
    security.declareProtected('Manage SQLEngine', 'manage_loadSQL')    
    def manage_loadSQL(self, source=None):
        """ loads up queries from the fs """
        #print 'start'

        # currently config holds classes
        # no reason it couldn't hold objs
        # this a ts concern. granted using
        # a global isn't exactly ts, but
        # changes to a config should
        # be down to during downtime
        
        global config
        
        f = config['FINDER']()
        l = config['LOADER']()
        b = config['BUILDER']()

        sources_map_resources={}
        query_names = {}
        resource_map_queries = {}
        query_resource_modtimes = {}
        resource_recurse=1
        
        if not source:
            sources = self._sources_map_resources.keys()
        else:
            sources = [source]
        
            
        for s in sources:
            #print 'source', s
            
            sources_map_resources[s]=[]
            
            query_resources = f.findQueryResources(s,
                                               resource_recurse)

            #print 'resources', query_resources

            for resource, mod_time in query_resources:
                query_resource_modtimes[resource]=mod_time
                query_dicts = l.loadQueries(resource)
            
                #print
                #print 'queries names'
            
                #for q in query_dicts:
                    #print '   ',q['name']
                
                resource_map_queries[resource]=[]
            
                for q in query_dicts:
                    qn = q['name']
                    resource_map_queries[resource].append(qn)

                    # need to switch to more human readable version
                    #print 'qname', qn
                    qid = str(abs(hash(qn)))
                    if hasattr(self.aq_base, qid):
                        if query_names.has_key(qn):
                            raise SyntaxError (
                                  "duplicate name %s in %s from %s"%(
                                                                     qn, 
                                                                     resource,
                                                                     s
                                                                     ))
                        else:
                            qid = qid + '1'
                    
                    qobj = b.build(self.conn_id, qid, q)
                    query_names[qn]=qid
                    self._setObject(query_names[qn], qobj)
                 
                sources_map_resources[s].append(resource)
                 
        # update attributes
        ## need some extra work to prevent collisions
        self._query_names.update(query_names)
        self._resource_map_queries.update(resource_map_queries)

        self._query_resource_modtimes.update(query_resource_modtimes)
        self._sources_map_resources.update(sources_map_resources)
        
        self._p_changed = 1

    security.declareProtected('Manage SQLEngine', 'clearSQL')
    def manage_clearSQL(self):
        """ removes all query objects """

        self.manage_delObjects(self._query_names.values())
        self._query_names.clear()
        self._resource_map_queries.clear()
        self._query_resource_modtimes.clear()
        
        sources=self._sources_map_resources.keys()
        self._sources_map_resources.clear()        
        for s in sources:
            self._sources_map_resources[s]=[]
        


    security.declareProtected('Manage SQLEngine', 'reloadSQL')
    def manage_reloadSQL(self):
        """ reloads all queries from fs """
        
        self.clearSQL()
        self.loadSQL()
        
    '''
    def refreshSQL(self):
        """ reloads all queries modified on fs """

        for resource, mod_time in self._resource_map_queries.items():
            
            if getmtime(resource) > mod_time:
                qns = self._resource_map_queries[resource]
                qids = map(self._query_names.get, qns)
                self.manage_delObjects(qids)
                del_qid = lambda q,n=self._query_names: del n[q]
                map(del_qid, qns)
                del self._resource_map_queries[resource]
    '''

Globals.InitializeClass(SQLEngine)
    

                        
                        
